@@ -1,4 +1,4 @@
-/*  Copyright (C) 2004 Alexander Neundorf <neundorf@kde.org>
+/*  Copyright (C) 2004-2005 Alexander Neundorf <neundorf@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <qprocess.h>
 #include <qfileinfo.h>
 #include <qregexp.h>
+#include <qspinbox.h>
 
 #include <iostream.h>
 
@@ -51,27 +52,35 @@
 
 void millisleep(int ms)
 {
-   struct timeval tv;
-   tv.tv_sec=0;
-   tv.tv_usec=ms*1000;
-   select(0, 0, 0, 0, &tv);
+   if (ms>0)
+   {
+      struct timeval tv;
+      tv.tv_sec=0;
+      tv.tv_usec=ms*1000;
+      select(0, 0, 0, 0, &tv);
+   }
 }
 
 QCPPDialogImpl::QCPPDialogImpl(QWidget* parent)
-   :CuteCommDlg(parent)
-   ,m_isConnected(false)
-   ,m_fd(-1)
-   ,m_cmdBufIndex(0)
-   ,m_notifier(0)
-   ,m_sz(0)
-   ,m_progress(0)
-   ,m_progressStepSize(1000)
+:CuteCommDlg(parent)
+,m_isConnected(false)
+,m_fd(-1)
+,m_cmdBufIndex(0)
+,m_notifier(0)
+,m_sz(0)
+,m_progress(0)
+,m_progressStepSize(1000)
 ,m_fileDlg(0)
+,m_outputTimer(this)
+,m_keyRepeatTimer(this)
+,m_keyCode(0)
+//,m_firstRep(true)
+,m_hexBytes(0)
 {
    connect(m_connectPb, SIGNAL(clicked()), this, SLOT(connectTTY()));
    connect(m_closePb, SIGNAL(clicked()), this, SLOT(disconnectTTY()));
    connect(m_clearOutputPb, SIGNAL(clicked()), m_outputView, SLOT(clear()));
-   connect(m_clearInputPb, SIGNAL(clicked()), m_oldCmdsLb, SLOT(clear()));
+//   connect(m_clearInputPb, SIGNAL(clicked()), m_oldCmdsLb, SLOT(clear()));
 
    connect(m_cmdLe, SIGNAL(returnPressed()), this, SLOT(execCmd()));
 
@@ -82,6 +91,8 @@ QCPPDialogImpl::QCPPDialogImpl(QWidget* parent)
    connect(m_oldCmdsLb, SIGNAL(clicked(QListBoxItem*)), this, SLOT(oldCmdClicked(QListBoxItem*)));
    connect(m_oldCmdsLb, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(execCmd()));
 
+   connect(m_hexOutputCb, SIGNAL(toggled(bool)), this, SLOT(hexOutputClicked(bool)));
+
    connect(m_connectPb, SIGNAL(clicked()), this, SLOT(saveSettings()));
 //   connect(m_deviceCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
    connect(m_baudCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
@@ -94,7 +105,19 @@ QCPPDialogImpl::QCPPDialogImpl(QWidget* parent)
    connect(m_readCb, SIGNAL(clicked()), this, SLOT(saveSettings()));
    connect(m_writeCb, SIGNAL(clicked()), this, SLOT(saveSettings()));
    connect(m_applyCb, SIGNAL(clicked()), this, SLOT(saveSettings()));
+   connect(m_hexOutputCb, SIGNAL(clicked()), this, SLOT(saveSettings()));
+   connect(m_inputModeCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
+   connect(m_charDelaySb, SIGNAL(valueChanged(int)), this, SLOT(saveSettings()));
+   connect(m_logAppendCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
+
    connect(m_applyCb, SIGNAL(toggled(bool)), this, SLOT(enableSettingWidgets(bool)));
+   connect(m_logFileFileDialog, SIGNAL(clicked()), this, SLOT(chooseLogFile()));
+
+   connect(&m_outputTimer, SIGNAL(timeout()), this, SLOT(doOutput()));
+   connect(&m_keyRepeatTimer, SIGNAL(timeout()), this, SLOT(sendKey()));
+
+   connect(m_enableLoggingCb, SIGNAL(toggled(bool)), this, SLOT(enableLogging(bool)));
+//   connect(m_enableLoggingCb, SIGNAL(toggled(bool)), this, SLOT(enableLogging(bool)));
 
    m_outputView->setWrapPolicy(QTextEdit::Anywhere);
    m_outputView->setWordWrap(QTextEdit::WidgetWidth);
@@ -105,6 +128,8 @@ QCPPDialogImpl::QCPPDialogImpl(QWidget* parent)
    accel->insertItem(CTRL+Key_S, 19);
    connect(accel, SIGNAL(activated(int)), this, SLOT(sendByte(int)));*/
 
+   m_outputTimerStart.start();
+
    readSettings();
 
    disconnectTTY();
@@ -137,6 +162,16 @@ void QCPPDialogImpl::saveSettings()
    settings.writeEntry("/cutecom/width", width());
    settings.writeEntry("/cutecom/height", height());
 
+   settings.writeEntry("/cutecom/LineMode", m_inputModeCb->currentItem());
+   settings.writeEntry("/cutecom/HexOutput", m_hexOutputCb->isChecked());
+   settings.writeEntry("/cutecom/CharDelay", m_charDelaySb->value());
+
+   settings.writeEntry("/cutecom/SendFileDialogStartDir", m_sendFileDialogStartDir);
+   settings.writeEntry("/cutecom/LogFileName", m_logFileLe->text());
+
+   settings.writeEntry("/cutecom/AppendToLogFile", m_logAppendCb->currentItem());
+
+
    QString currentDevice=m_deviceCb->currentText();
    settings.writeEntry("/cutecom/CurrentDevice", currentDevice);
    bool currentDeviceIsInList=false;
@@ -181,6 +216,14 @@ void QCPPDialogImpl::readSettings()
    m_stopCb->setCurrentItem(settings.readNumEntry("/cutecom/Stopbits", 0));
    m_protoPb->setCurrentItem(settings.readNumEntry("/cutecom/Protocol", 0));
 
+   m_inputModeCb->setCurrentItem(settings.readNumEntry("/cutecom/LineMode", 0));
+   m_hexOutputCb->setChecked(settings.readBoolEntry("/cutecom/HexOutput", false));
+   m_charDelaySb->setValue(settings.readNumEntry("/cutecom/CharDelay", 1));
+
+   m_sendFileDialogStartDir=settings.readEntry("/cutecom/SendFileDialogStartDir", QDir::homeDirPath());
+   m_logFileLe->setText(settings.readEntry("/cutecom/LogFileName", QDir::homeDirPath()+"/cutecom.log"));
+
+   m_logAppendCb->setCurrentItem(settings.readNumEntry("/cutecom/AppendToLogFile", 0));
 
    int x=settings.readNumEntry("/cutecom/width", -1);
    int y=settings.readNumEntry("/cutecom/height", -1);
@@ -209,23 +252,30 @@ void QCPPDialogImpl::readSettings()
 
 void QCPPDialogImpl::showAboutMsg()
 {
-   QMessageBox::about(this, tr("About CuteCom"), tr("This is CuteCom 0.11.0<br>(c)2004 Alexander Neundorf, &lt;neundorf@kde.org&gt;<br>Licensed under the GNU GPL v2"));
+   QMessageBox::about(this, tr("About CuteCom"), tr("This is CuteCom 0.14.0<br>(c)2004-2006 Alexander Neundorf, &lt;neundorf@kde.org&gt;<br>Licensed under the GNU GPL v2"));
 }
 
 void QCPPDialogImpl::sendFile()
 {
    if (m_fileDlg==0)
    {
-      m_fileDlg=new QFileDialog(QDir::homeDirPath());
+      m_fileDlg=new QFileDialog(m_sendFileDialogStartDir);
       m_fileDlg->setMode(QFileDialog::ExistingFile);
    }
 
    QString filename;
    if ( m_fileDlg->exec() == QDialog::Accepted )
+   {
       filename = m_fileDlg->selectedFile();
+      m_sendFileDialogStartDir=m_fileDlg->dirPath();
+      saveSettings();
+   }
    else
+   {
       return;
+   }
 
+   unsigned int charDelay=m_charDelaySb->value();
    if (m_protoPb->currentText()=="Script")
    {
       QFile file(filename);
@@ -248,6 +298,7 @@ void QCPPDialogImpl::sendFile()
             QMessageBox::information(this, tr("Comm error"), tr("Sending failed"));
             return;
          }
+         millisleep(charDelay*3);
       }
    }
    else if (m_protoPb->currentText()=="Plain")
@@ -272,7 +323,9 @@ void QCPPDialogImpl::sendFile()
             m_progress->setProgress(i/step);
             qApp->processEvents();
          }
-         sendByte(data.data()[i]);
+         sendByte(data.data()[i], charDelay);
+         if ((data.data()[i]=='\n') || (data.data()[i]=='\r')) //wait twice as long after bytes which might be line ends (helps some uCs)
+            millisleep(charDelay);
 //         if (!sendByte(data.data()[i]))
          if (0)
          {
@@ -440,24 +493,51 @@ bool QCPPDialogImpl::eventFilter(QObject* watched, QEvent *e)
       {
          if (ke->key()==Key_C)
          {
-            sendByte(3);
+//            cerr<<"c";
+            m_keyCode=3;
+            sendByte(m_keyCode, 0);
+//            if (m_firstRep)
+               m_keyRepeatTimer.start(0, false);
             return true;
          }
          else if (ke->key()==Key_Q)
          {
-            sendByte(17);
+//            cerr<<"#";
+            m_keyCode=17;
+            sendByte(m_keyCode, 0);
+//            m_keyRepeatTimer.start(200, true);
             return true;
          }
          else if (ke->key()==Key_S)
          {
-            sendByte(19);
+            m_keyCode=19;
+            sendByte(m_keyCode, 0);
+//            m_keyRepeatTimer.start(200, true);
             return true;
          }
       }
    }
+//   else if ((watched==m_cmdLe) && (e->type()==QEvent::KeyRelease) && (ke->key()==Key_Control))
+   else if ((watched==m_cmdLe) && (e->type()==QEvent::KeyRelease))
+   {
+      m_keyRepeatTimer.stop();
+//      m_firstRep=true;
+   }
+
    return false;
 }
 
+void QCPPDialogImpl::sendKey()
+{
+//   cerr<<"-";
+   sendByte(m_keyCode, 0);
+/*   if (m_firstRep)
+   {
+      m_firstRep=false;
+      m_keyRepeatTimer.start(1, false);
+   }*/
+}
+
 void QCPPDialogImpl::oldCmdClicked(QListBoxItem* item)
 {
    if (item==0)
@@ -547,20 +627,70 @@ void QCPPDialogImpl::execCmd()
 
 bool QCPPDialogImpl::sendString(const QString& s)
 {
+   unsigned int lineMode=m_inputModeCb->currentItem();
+   unsigned int charDelay=m_charDelaySb->value();
+
+   if (lineMode==4) // hex
+   {
+      QString hex=s;
+      hex.remove(QRegExp("\\s"));
+      if ((hex.startsWith("0x")) || (hex.startsWith("0X")))
+         hex=hex.mid(2);
+
+      if (hex.length()%2 != 0)
+         hex="0"+hex;
+
+      for (unsigned int i=0; i<hex.length()/2; i++)
+      {
+         QString nextByte=hex.mid(i*2, 2);
+         bool ok=true;
+         nextByte.toUInt(&ok, 16);
+         if (!ok)
+         {
+            QMessageBox::information(this, tr("Invalid hex characters"), tr("The input string contains invalid hex characters: 0x%1").arg(nextByte));
+            return false;
+         }
+      }
+
+      for (unsigned int i=0; i<hex.length()/2; i++)
+      {
+         QString nextByte=hex.mid(i*2, 2);
+         unsigned int byte=nextByte.toUInt(0, 16);
+         sendByte(byte & 0xff, charDelay);
+         fprintf(stderr, " 0x%x d:%d ", byte & 0xff, charDelay);
+      }
+      return true;
+   }
+
    const char *bytes=s.latin1();
    for (unsigned int i=0; i<s.length(); i++)
    {
-      if (!sendByte(*bytes))
+      if (!sendByte(*bytes, charDelay))
          return false;
       bytes++;
-      millisleep(1);
    }
-   if (!sendByte('\n'))
-      return false;
+
+   if (lineMode==0)
+   {
+      if (!sendByte('\n', charDelay))
+         return false;
+   }
+   else if (lineMode==1)
+   {
+      if (!sendByte('\r', charDelay))
+         return false;
+   }
+   else if (lineMode==2)
+   {
+      if (!sendByte('\r', charDelay))
+         return false;
+      if (!sendByte('\n', charDelay))
+         return false;
+   }
    return true;
 }
 
-bool QCPPDialogImpl::sendByte(char c)
+bool QCPPDialogImpl::sendByte(char c, unsigned int delay)
 {
    if (m_fd==-1)
       return false;
@@ -573,6 +703,7 @@ bool QCPPDialogImpl::sendByte(char c)
       perror("write\n");
       return false;
    }
+   millisleep(delay);
 //   else
 //      cerr<<" \""<<c<<"\"; ";
    return true;
@@ -620,6 +751,9 @@ void QCPPDialogImpl::connectTTY()
       return;
    }
 
+   // flushing is to be done after opening. This prevents first read and write to be spam'ish.
+   tcflush(m_fd, TCIOFLUSH);
+
    if (m_applyCb->isChecked())
    {
       int n = fcntl(m_fd, F_GETFL, 0);
@@ -674,6 +808,9 @@ void QCPPDialogImpl::disconnectTTY()
 
 void QCPPDialogImpl::disconnectTTYRestore(bool restoreSettings)
 {
+   m_outputTimer.stop();
+   m_outputBuffer="";
+
 //   cerr<<"closing "<<m_fd<<endl;
    if (m_fd!=-1)
    {
@@ -849,7 +986,7 @@ void QCPPDialogImpl::setNewOptions(int baudrate, int databits, const QString& pa
    newtio.c_oflag=0;
 
 
-   newtio.c_cc[VTIME]=2;
+   newtio.c_cc[VTIME]=1;
    newtio.c_cc[VMIN]=60;
 
 //   tcflush(m_fd, TCIFLUSH);
@@ -917,25 +1054,51 @@ void QCPPDialogImpl::readData(int fd)
       return;
    }
 
+   if (m_logFile.isOpen())
+   {
+      m_logFile.writeBlock(m_buf, bytesRead);
+   }
+
    QString text;
+   char buf[16];
    for (int i=0; i<bytesRead; i++)
    {
-      if ((isprint(*c)) || (*c=='\n') || (*c=='\r'))
+      if (m_hexOutputCb->isChecked())
       {
-         if (*c!='\r')
-            text+=(*c);
+         if ((m_hexBytes % 16) == 0)
+         {
+            snprintf(buf, 16, "%08x: ", m_hexBytes);
+            text+=buf;
+         }
+         unsigned int b=*c;
+         snprintf(buf, 16, "%02x ", b & 0xff);
+         text+=buf;
+
+         m_hexBytes++;
+         if ((m_hexBytes % 16)==0)
+            text+="\n";
+         else if ((m_hexBytes % 8)==0)
+            text+="  ";
       }
       else
       {
-         char buf[16];
-         unsigned int i=*c;
-         snprintf(buf, 16, "\\0x%02x", i & 0xff);
-         text+=buf;
+         if ((isprint(*c)) || (*c=='\n') || (*c=='\r'))
+         {
+            if (*c!='\r')
+               text+=(*c);
+         }
+         else
+         {
+            unsigned int b=*c;
+            snprintf(buf, 16, "\\0x%02x", b & 0xff);
+            text+=buf;
+         }
       }
       c++;
    }
 
-   if (m_outputView->paragraphs()>1100)
+   addOutput(text);
+/*   if (m_outputView->paragraphs()>1100)
    {
       m_outputView->setUpdatesEnabled(false);
       m_outputView->setSelection(0, 0, 100, 0);
@@ -949,7 +1112,111 @@ void QCPPDialogImpl::readData(int fd)
    {
       m_outputView->setCursorPosition(m_outputView->paragraphs()-1, m_outputView->paragraphLength(m_outputView->paragraphs()-1));
       m_outputView->insert(text);
+   }*/
+}
+
+void QCPPDialogImpl::addOutput(const QString& text)
+{
+   m_outputBuffer+=text;
+
+   if (!m_outputTimer.isActive())
+   {
+      doOutput();
+      m_outputTimerStart.restart();
+      m_outputTimer.start(50, true);
+   }
+   else
+   {
+      if ((m_outputTimerStart.elapsed()>400)
+         || ((m_outputTimerStart.elapsed()>200) && (m_outputBuffer.length()<100)))
+      {
+         doOutput();
+         m_outputTimerStart.restart();
+      }
+      m_outputTimer.start(50, true);
    }
 }
 
+void QCPPDialogImpl::doOutput()
+{
+   if (m_outputBuffer.isEmpty())
+      return;
+//   cerr<<"*";
+   int pNumber=m_outputView->paragraphs();
+   if (pNumber>1100)
+   {
+      m_outputView->setUpdatesEnabled(false);
+      m_outputView->setSelection(0, 0, pNumber-1000, 0);
+      m_outputView->removeSelectedText();
+      m_outputView->scrollToBottom();
+      m_outputView->setCursorPosition(m_outputView->paragraphs()-1, m_outputView->paragraphLength(m_outputView->paragraphs()-1));
+      m_outputView->insert(m_outputBuffer);
+      m_outputView->setUpdatesEnabled(true);
+   }
+   else
+   {
+      m_outputView->setCursorPosition(m_outputView->paragraphs()-1, m_outputView->paragraphLength(m_outputView->paragraphs()-1));
+      m_outputView->insert(m_outputBuffer);
+   }
+   m_outputBuffer="";
+}
+
+void QCPPDialogImpl::hexOutputClicked(bool on)
+{
+   addOutput("\n");
+   m_hexBytes=0;
+}
+
+void QCPPDialogImpl::enableLogging(bool on)
+{
+   if (m_logFile.isOpen()==on)
+      return;
+
+   if (on)
+   {
+      m_logFile.setName(m_logFileLe->text());
+      int mode=IO_ReadWrite;
+      if (m_logAppendCb->currentItem()==0)
+      {
+         mode=mode | IO_Truncate;
+      }
+      else
+      {
+         mode=mode | IO_Append;
+      }
+
+      if (!m_logFile.open(mode))
+      {
+         QMessageBox::information(this, tr("Opening file failed"), tr("Could not open file %1 for writing").arg(m_logFileLe->text()));
+         m_enableLoggingCb->setChecked(false);
+      }
+      else
+      {
+         m_logAppendCb->setEnabled(false);
+         m_logFileLe->setEnabled(false);
+         m_logFileFileDialog->setEnabled(false);
+         saveSettings();
+      }
+   }
+   else
+   {
+      m_logFile.close();
+
+      m_logAppendCb->setEnabled(true);
+      m_logFileLe->setEnabled(true);
+      m_logFileFileDialog->setEnabled(true);
+
+   }
+
+}
+
+
+void QCPPDialogImpl::chooseLogFile()
+{
+   QString logFile=QFileDialog::getSaveFileName(m_logFileLe->text());
+   if (!logFile.isEmpty())
+   {
+      m_logFileLe->setText(logFile);
+   }
+}
 
